(**************************************************************)
(*
 *  Ensemble, (Version 0.61)
 *  Copyright 1999 Cornell University
 *  All rights reserved.
 *
 *  See ensemble/doc/license.txt for further information.
 *)
(**************************************************************)
(**************************************************************)
(* CHK_SECCHAN.ML *)
(* Author: Ohad Rodeh, 3/99 *)
(* Tests for the secchan/mngchan layers *)
(**************************************************************)
open Layer
open Trans
open Util
open View
open Event
(**************************************************************)
let name = Trace.filel "CHK_SECCHAN"
(**************************************************************)

type state = {
  nonce : int ;
  ack   : bool array ;
  mutable first_time : bool
}

type headr = NoHdr

type msg = Out of int | In of int

let init s (ls,vs) = {
  nonce = ls.rank ;
  ack = Array.init ls.nmembers (function i -> i=ls.rank);
  first_time = true 
}

(**************************************************************)

let marshal, unmarshal = Buf.make_marsh "CHK_SECCHAN" true

let check_all a = array_for_all ident a 

let hdlrs s ((ls,vs) as vf) {up_out=up;upnm_out=upnm;dn_out=dn;dnlm_out=dnlm;dnnm_out=dnnm} =
  let log = Trace.log2 name ls.name in
  let loga = Trace.log2 (name^"A") ls.name in
  let failwith m = failwith ("("^name^":"^m^")") in

  let up_hdlr ev abv hdr = match getType ev,hdr with
    | _ -> up ev abv

  and uplm_hdlr ev hdr = match getType ev,hdr with
	  (* Echo received messages *)
    | ESecureMsg,NoHdr -> 
	let src = getPeer ev in
	let m = getSecureMsg ev in
	let m = unmarshal m Buf.len0 (Buf.length m) in
	begin match m with 
	  | Out r -> 
	      loga (fun () -> sprintf "Got [%d] msg=%d" src r);
	      let reply = marshal (In r) in
	      dnlm (create name ESecureMsg [Peer src; SecureMsg reply]) NoHdr
	  | In r -> 
	      if r = s.nonce then (
		loga (fun () -> sprintf "Echo from %d -- OK" src);
		s.ack.(src) <- true ;
		if (check_all s.ack) then 
		  log (fun () -> "All OK")
	      ) else 
		failwith (sprintf "Echo from %d [val=%d]" src r)
	end

    | _ -> failwith unknown_local

  and upnm_hdlr ev = match getType ev with
    | ETimer -> 
	if s.first_time && ls.nmembers > 1 then (
	  s.first_time <- false ;
	  log (fun () -> sprintf "sending %d" s.nonce);
	  let rnd = marshal (Out s.nonce) in
	  let a = Array.init ls.nmembers (function i -> i<>ls.rank) in
	  Array.iteri (fun i v -> 
	    if v then (
	      dnlm (create name ESecureMsg [Peer i; SecureMsg rnd]) NoHdr
	    )
	  ) a
	);
	upnm ev	

    | _ -> upnm ev

  and dn_hdlr ev abv = match getType ev with
    | _ -> dn ev abv NoHdr

  and dnnm_hdlr = dnnm

in {up_in=up_hdlr;uplm_in=uplm_hdlr;upnm_in=upnm_hdlr;dn_in=dn_hdlr;dnnm_in=dnnm_hdlr}

let l args vf = Layer.hdr init hdlrs None NoOpt args vf

let _ = Elink.layer_install name l

(**************************************************************)




